@@ -46,7 +46,8 @@ CG_inflator <- function(x = 1, from_fy, to_fy, forecast.series = "mean"){
   stopifnot(is.numeric(x), all(is.fy(from_fy)), all(is.fy(to_fy)))
   
   # Else NAs.
-  stopifnot(all(to_fy %in% cgt_expenditures$FY), all(from_fy %in% cgt_expenditures$FY))
+  stopifnot(all(to_fy %in% cg_inflators_1314[["fy_year"]]),
+            all(from_fy %in% cg_inflators_1314[["fy_year"]]))
 
   # CRAN Note avoidance
   ordering <- NULL
