@@ -10,6 +10,7 @@
 #' @param return.mode The mode (numeric or integer) of the returned vector.
 #' @param .dots.ATO A data.frame that contains additional information about the individual's circumstances, with columns the same as in the ATO sample files. If \code{.dots.ATO} is a \code{data.table}, I recommend you enclose it with \code{copy()}.
 #' @param allow.forecasts should dates beyond 2019-20 be permitted? Currently, not permitted.
+#' @param .debug (logical, default: \code{FALSE})  If \code{TRUE}, returns a \code{data.table} containing the components. (This argument and its result is liable to change in future versions, possibly without notice.)
 #' @author Tim Cameron, Brendan Coates, Hugh Parsonage, William Young
 #' @details The function 'rolling' is inflexible by design. It is designed to guarantee the correct tax payable in a year.
 #' For years preceding the introduction of SAPTO, the maximum offset is assumed to apply to those above pensionable age. 
@@ -36,7 +37,8 @@ income_tax <- function(income,
                        n_dependants = 0L,
                        .dots.ATO = NULL,
                        return.mode = c("numeric", "integer"),
-                       allow.forecasts = FALSE) {
+                       allow.forecasts = FALSE,
+                       .debug = FALSE) {
   if (missing(fy.year)){
     stop("fy.year is missing, with no default")
   }
@@ -51,12 +53,20 @@ income_tax <- function(income,
     }
   }
   
-  if (identical(fy.year, "2013-14") &&
-      is.null(age)) {
+  # Absolutely necessary
+  .dots.ATO <- copy(.dots.ATO)
+  
+  if (is.null(age) &&
+      length(fy.year) == 1L &&
+      fy.year %chin% c("2012-13", "2013-14", 
+                       "2014-15", "2015-16",
+                       "2016-17", "2017-18",
+                       "2018-19", "2019-20")) {
     out <- income_tax_cpp(income,
-                          fy.year = "2013-14",
+                          fy.year = fy.year,
                           .dots.ATO = .dots.ATO,
-                          n_dependants = n_dependants)
+                          n_dependants = n_dependants,
+                          .debug = .debug)
   } else {
     out <- rolling_income_tax(income = income,
                               fy.year = fy.year,
@@ -64,7 +74,8 @@ income_tax <- function(income,
                               family_status = family_status,
                               n_dependants = n_dependants, 
                               .dots.ATO = .dots.ATO,
-                              allow.forecasts = allow.forecasts)
+                              allow.forecasts = allow.forecasts, 
+                              .debug = .debug)
   }
   return.mode <- match.arg(return.mode)
   
@@ -85,7 +96,8 @@ rolling_income_tax <- function(income,
                                family_status = "individual",
                                n_dependants = 0L,
                                .dots.ATO = NULL,
-                               allow.forecasts = FALSE) {
+                               allow.forecasts = FALSE, 
+                               .debug = FALSE) {
   
   if (!all(fy.year %chin% c("2000-01", "2001-02", 
                             "2002-03", "2003-04", "2004-05", "2005-06", "2006-07", "2007-08", 
@@ -105,9 +117,7 @@ rolling_income_tax <- function(income,
     }
   }
   
-  if (any(income < 0)){
-    warning("Negative entries in income detected. These will have value NA.")
-  }
+
   
   # CRAN NOTE avoidance
   fy_year <- NULL; marginal_rate <- NULL; lower_bracket <- NULL; tax_at <- NULL; n <- NULL; tax <- NULL; ordering <- NULL; max_lito <- NULL; min_bracket <- NULL; lito_taper <- NULL; sato <- NULL; taper <- NULL; rate <- NULL; max_offset <- NULL; upper_threshold <- NULL; taper_rate <- NULL; medicare_income <- NULL; lower_up_for_each_child <- NULL;
@@ -133,13 +143,7 @@ rolling_income_tax <- function(income,
         sapto.eligible <- coalesce(between(.subset2(.dots.ATO, "age_range"), 0L, 1L), FALSE)  
       }
     } else {
-      if ("Birth_year" %chin% .dots.ATO.noms) {
-        sapto.eligible <- coalesce(between(.subset2(.dots.ATO, "Birth_year"), 0L, 1L), FALSE)
-      } else {
-        stop("`.dots.ATO` was supplied, but did not contain columns 'age_range' or 'Birth_year'. ",
-             "`dots.ATO` needs to be a sample file with the original names. Ensure `.dots.ATO` ",
-             "has either of those names.")
-      }
+      sapto.eligible <- coalesce(between(.subset2(.dots.ATO, "Birth_year"), 0L, 1L), FALSE)
     }
   }
   
@@ -152,6 +156,7 @@ rolling_income_tax <- function(income,
   prohibit_vector_recycling(income, fy.year, age, family_status, n_dependants)
   prohibit_length0_vectors(income, fy.year, age, family_status, n_dependants)
   
+  # Also lengths() -- but doesn't appear to be faster, despite being so advertised 
   input.lengths <- vapply(list(income, fy.year, age, family_status, n_dependants), FUN = length, FUN.VALUE = integer(1))
   max.length <- max(input.lengths)
   
@@ -245,10 +250,42 @@ rolling_income_tax <- function(income,
   
   # https://www.legislation.gov.au/Details/C2014A00048
   # input[["fy_year"]] ensures it matches the length of income if length(fy.year) == 1.
-  temp_budget_repair_levy. <- (input[["fy_year"]] %chin% c("2014-15", "2015-16", "2016-17") & income > 180e3) * (0.02 * (income - 180e3))
+  if (any(c("2014-15", "2015-16", "2016-17") %fin% .subset2(input, "fy_year"))) {
+    temp_budget_repair_levy. <-
+      0.02 * pmaxC(income - 180e3, 0) *
+      {.subset2(input, "fy_year") %chin% c("2014-15", "2015-16", "2016-17")}
+  } else {
+    temp_budget_repair_levy. <- 0
+  }
   
-  pmaxC(base_tax. - lito. - sapto., 
-        0) + medicare_levy. + temp_budget_repair_levy.
+  if (any("2011-12" %fin% .subset2(input, "fy_year"))) {
+    flood_levy. <- 
+      0.005 *
+      {.subset2(input, "fy_year") == "2011-12"} * 
+      {pmaxC(income - 50e3, 0) + pmaxC(income - 100e3, 0)}
+  } else {
+    flood_levy. <- 0
+  }
+  
+  # http://classic.austlii.edu.au/au/legis/cth/consol_act/itaa1997240/s4.10.html
+  S4.10_basic_income_tax_liability <- pmaxC(base_tax. - lito. - sapto., 0)
+  
+  # SBTO can only be calculated off .dots.ATO
+  if (is.null(.dots.ATO)) {
+    sbto. <- 0
+  } else {
+    sbto. <-
+      small_business_tax_offset(taxable_income = income,
+                                basic_income_tax_liability = S4.10_basic_income_tax_liability,
+                                .dots.ATO = .dots.ATO,
+                                fy_year = fy.year)
+  }
+  
+  # SBTO is non-refundable (Para 1.6 of explanatory memo)
+  pmaxC(S4.10_basic_income_tax_liability - sbto., 0) +
+    medicare_levy. +
+    temp_budget_repair_levy. +
+    flood_levy.
 }
 
 
@@ -256,7 +293,7 @@ rolling_income_tax <- function(income,
 
 
 
-income_tax_cpp <- function(income, fy.year, .dots.ATO = NULL, sapto.eligible = NULL, n_dependants = 0L) {
+income_tax_cpp <- function(income, fy.year, .dots.ATO = NULL, sapto.eligible = NULL, n_dependants = 0L, .debug = FALSE) {
   # Assume everyone of pension age is eligible for sapto.
   .dots.ATO.noms <- names(.dots.ATO)
   
@@ -301,88 +338,179 @@ income_tax_cpp <- function(income, fy.year, .dots.ATO = NULL, sapto.eligible = N
     n_dependants <- rep_len(n_dependants, max.length)
   }
   
+  if (length(fy.year) != 1L) {
+    stop("`fy.year` was length-",
+         prettyNum(length(fy.year), big.mark = ","), ". ",
+         "Use a single-length financial year in `income_tax_cpp()` ",
+         "or use `income_tax()`.",
+         call. = FALSE)
+  }
+  
+  if (fy.year < "2012-13") {
+    stop("`fy.year` was ", fy.year, ". ",
+         "Use a single financial year from 2012-13 onwards ", 
+         "or use `income_tax()`.", 
+         call. = FALSE)
+  }
+  
+  base_tax. <- 
+    IncomeTax(income, 
+              thresholds = c(0, 18200, 37000,
+                             if (fy.year < "2016-17") 80e3 else 87e3,
+                             180000),
+              rates = c(0, 0.19, 0.325, 0.37, 
+                        # Temp budget repair levy
+                        if (OR(fy.year == "2014-15",
+                               OR(fy.year == "2015-16",
+                                  fy.year == "2016-17"))) 0.47 else 0.45))
+  
+  lito. <- pminC(pmaxC(445 - (income - 37000) * 0.015, 0),
+                 445)
 
   switch(fy.year,
+         "2012-13" = {
+           Year.int <- 2013L
+         },
          "2013-14" = {
-           base_tax. <- 
-             IncomeTax(income, 
-                       thresholds = c(0, 18200, 37000, 80000, 180000),
-                       rates = c(0, 0.19, 0.325, 0.37, 0.45))
-           
            Year.int <- 2014L
-           
-           if (any(sapto.eligible)) {
-             # Use this a lot, saves 0.5ms each time
-             which_sapto <- which(sapto.eligible)
-             which_not_sapto <- which(!sapto.eligible)
-             
-             SpouseIncome_sapto_eligible <- SpouseIncome[which_sapto]
-             
-             if (length(sapto.eligible) != 1L) {
-               medicare_levy. <- double(max.length)
-               medicare_levy.[which_sapto] <- 
-                 MedicareLevySaptoYear(income[which_sapto],
-                                       SpouseIncome = SpouseIncome_sapto_eligible,
-                                       NDependants = n_dependants[which_sapto], 
-                                       TRUE, 
-                                       Year.int)
-               
-               medicare_levy.[which_not_sapto] <- 
-                 MedicareLevySaptoYear(income[which_not_sapto],
-                                       SpouseIncome = SpouseIncome[which_not_sapto],
-                                       NDependants = n_dependants[which_not_sapto],
-                                       FALSE,
-                                       Year.int)
-               
-             } else {
-               # All are SAPTO.
-               medicare_levy. <-
-                 MedicareLevySaptoYear(income,
-                                       SpouseIncome = SpouseIncome,
-                                       NDependants = n_dependants,
-                                       TRUE, 
-                                       Year.int)
-             }
-             
-             if (AND(!is.null(.dots.ATO),
-                     all(c("Rptbl_Empr_spr_cont_amt",
-                           "Net_fincl_invstmt_lss_amt",
-                           "Net_rent_amt", 
-                           "Rep_frng_ben_amt") %chin% .dots.ATO.noms))) {
-               sapto. <- double(max.length)
-               .dAse <- .dots.ATO[which_sapto]
-               
-               sapto.[which_sapto] <-
-                 sapto_rcpp_yr(RebateIncome = rebate_income(Taxable_Income = income[which_sapto],
-                                                            Rptbl_Empr_spr_cont_amt = .dAse[["Rptbl_Empr_spr_cont_amt"]],
-                                                            Net_fincl_invstmt_lss_amt = .dAse[["Net_fincl_invstmt_lss_amt"]],
-                                                            Net_rent_amt = .dAse[["Net_rent_amt"]],
-                                                            Rep_frng_ben_amt = .dAse[["Rep_frng_ben_amt"]]),
-                               SpouseIncome = SpouseIncome_sapto_eligible,
-                               IsMarried = SpouseIncome_sapto_eligible > 0,
-                               yr = Year.int)
-
-             } else {
-               sapto. <- sapto.eligible * sapto(rebate_income = rebate_income(Taxable_Income = income), 
-                                                fy.year = fy.year, 
-                                                sapto.eligible = TRUE)
-             }
-           } else {
-             medicare_levy. <-
-               MedicareLevySaptoYear(income = income,
-                                     SpouseIncome = SpouseIncome,
-                                     NDependants = n_dependants,
-                                     FALSE,
-                                     Year.int)
-             sapto. <- 0
-           }
-           
-           lito. <- pminC(pmaxC(445 - (income - 37000) * 0.015, 0),
-                          445)
-           
-           pmaxC(base_tax. - lito. - sapto., 0) + medicare_levy.
-             
-         })
+         },
+         "2014-15" = {
+           Year.int <- 2015L
+         },
+         "2015-16" = {
+           Year.int <- 2016L
+         },
+         "2016-17" = {
+           Year.int <- 2017L
+         },
+         "2017-18" = {
+           Year.int <- 2018L
+         },
+         "2018-19" = {
+           Year.int <- 2019L
+         }, 
+         "2019-20" = {
+           Year.int <- 2020L
+         },
+         stop("`fy.year` was ", fy.year, ". ",
+              "Use a single financial year from 2012-13 onwards ", 
+              "or use `income_tax()`.", 
+              call. = FALSE))
+  
+  if (any(sapto.eligible)) {
+    # Use this a lot, saves 0.5ms each time
+    which_sapto <- which(sapto.eligible)
+    which_not_sapto <- which(!sapto.eligible)
+    
+    SpouseIncome_sapto_eligible <- SpouseIncome[which_sapto]
+    
+    if (length(sapto.eligible) != 1L) {
+      medicare_levy. <- double(max.length)
+      medicare_levy.[which_sapto] <- 
+        MedicareLevySaptoYear(income[which_sapto],
+                              SpouseIncome = SpouseIncome_sapto_eligible,
+                              NDependants = n_dependants[which_sapto], 
+                              TRUE, 
+                              Year.int)
+      
+      medicare_levy.[which_not_sapto] <- 
+        MedicareLevySaptoYear(income[which_not_sapto],
+                              SpouseIncome = SpouseIncome[which_not_sapto],
+                              NDependants = n_dependants[which_not_sapto],
+                              FALSE,
+                              Year.int)
+      
+    } else {
+      # All are SAPTO.
+      medicare_levy. <-
+        MedicareLevySaptoYear(income,
+                              SpouseIncome = SpouseIncome,
+                              NDependants = n_dependants,
+                              TRUE, 
+                              Year.int)
+    }
+    
+    
+    
+    if (AND(!is.null(.dots.ATO),
+            all(c("Rptbl_Empr_spr_cont_amt",
+                  "Net_fincl_invstmt_lss_amt",
+                  "Net_rent_amt", 
+                  "Rep_frng_ben_amt") %chin% .dots.ATO.noms))) {
+      sapto. <- double(max.length)
+      .dAse <- .dots.ATO[which_sapto]
+      
+      sapto.[which_sapto] <-
+        sapto_rcpp_yr(RebateIncome = rebate_income(Taxable_Income = income[which_sapto],
+                                                   Rptbl_Empr_spr_cont_amt = .dAse[["Rptbl_Empr_spr_cont_amt"]],
+                                                   Net_fincl_invstmt_lss_amt = .dAse[["Net_fincl_invstmt_lss_amt"]],
+                                                   Net_rent_amt = .dAse[["Net_rent_amt"]],
+                                                   Rep_frng_ben_amt = .dAse[["Rep_frng_ben_amt"]]),
+                      SpouseIncome = SpouseIncome_sapto_eligible,
+                      IsMarried = SpouseIncome_sapto_eligible > 0,
+                      yr = Year.int)
+      
+    } else {
+      sapto. <- sapto.eligible * sapto(rebate_income = rebate_income(Taxable_Income = income), 
+                                       fy.year = fy.year, 
+                                       sapto.eligible = TRUE)
+    }
+  } else {
+    medicare_levy. <-
+      MedicareLevySaptoYear(income = income,
+                            SpouseIncome = SpouseIncome,
+                            NDependants = n_dependants,
+                            FALSE,
+                            Year.int)
+    sapto. <- 0
+  }
+  
+  # 2011-12 is never used
+  flood_levy. <- 0
+  
+  # http://classic.austlii.edu.au/au/legis/cth/consol_act/itaa1997240/s4.10.html
+  S4.10_basic_income_tax_liability <- pmaxC(base_tax. - lito. - sapto., 0)
+  
+  # SBTO can only be calculated off .dots.ATO
+  if (is.null(.dots.ATO)) {
+    sbto. <- 0
+  } else {
+    sbto. <-
+      small_business_tax_offset(taxable_income = income,
+                                basic_income_tax_liability = S4.10_basic_income_tax_liability,
+                                .dots.ATO = .dots.ATO,
+                                fy_year = fy.year)
+  }
+  
+  out <-
+    pmaxC(S4.10_basic_income_tax_liability - sbto., 0) +
+    medicare_levy. +
+    flood_levy.
+  
+  if (any(income < 0)) {
+    warning("Negative entries in income detected. These will have value NA.")
+    out[income < 0] <- switch(storage.mode(out), 
+                              "integer" = NA_integer_,
+                              "double" = NA_real_)
+  }
+  
+  if (.debug && is.data.table(.dots.ATO)) {
+    result <- 
+      copy(.dots.ATO) %>%
+      .[, "base_tax" := base_tax.] %>%
+      .[, "lito" := lito.] %>%
+      .[, "sapto" := sapto.] %>%
+      .[, "medicare_levy" := medicare_levy.] %>%
+      .[, "income_tax" := out]
+      
+    # if (fy.year == "2011-12") {
+    #   result[, "flood_levy" := flood_levy.]
+    # }
+    
+    return(result[])
+  }
+  
+  out
 }
 
 
