@@ -2,42 +2,89 @@
 #' 
 #' @param sample_file A sample file, most likely the 2012-13 sample file. It is intended that to be the most recent.
 #' @param h An integer. How many years should the sample file be projected?
-#' @param fy.year.of.sample.file The financial year of \code{sample_file}.
+#' @param fy.year.of.sample.file The financial year of \code{sample_file}. If \code{NULL}, the default, the number is inferred from the 
+#' number of rows of \code{sample_file} to be one of \code{2012-13}, \code{2013-14}, or \code{2014-15}.
 #' @param WEIGHT The sample weight for the sample file. (So a 2\% file has \code{WEIGHT} = 50.)
 #' @param excl_vars A character vector of column names in \code{sample_file} that should not be inflated. Columns not present in the 2013-14 sample file are not inflated and nor are the columns \code{Ind}, \code{Gender}, \code{age_range}, \code{Occ_code}, \code{Partner_status}, \code{Region}, \code{Lodgment_method}, and \code{PHI_Ind}.
 #' @param forecast.dots A list containing parameters to be passed to \code{generic_inflator}.
-#' @param wage.series See \code{\link{wage_inflator}}.
+#' @param wage.series See \code{\link{wage_inflator}}. Note that the \code{Sw_amt} will uprated by \code{\link{differentially_uprate_wage}}.
 #' @param lf.series See \code{\link{lf_inflator_fy}}.
-#' @param .recalculate.inflators Should \code{generic_inflator()} or \code{CG_inflator} be called to project the other variables? Adds time.
-#' @param .copyDT Should a \code{copy()} of \code{sample_file} be made? If set to FALSE, will update \code{sample_file}. 
+#' @param .recalculate.inflators (logical, default: \code{FALSE}. Should \code{generic_inflator()} or \code{CG_inflator} be called to project the other variables? Adds time.
+#' @param .copyDT (logical, default: \code{TRUE}) Should a \code{copy()} of \code{sample_file} be made? If set to \code{FALSE}, will update \code{sample_file}. 
+#' @param check_fy_sample_file (logical, default: \code{TRUE}) Should \code{fy.year.of.sample.file} be checked against \code{sample_file}?
+#' By default, \code{TRUE}, an error is raised if the base is not 2012-13, 2013-14, or 2014-15 and a warning is raised if the 
+#' number of rows in \code{sample_file} is different to the known number of rows in the sample files. 
 #' @return A sample file of the same number of rows as \code{sample_file} with inflated values (including WEIGHT).
-#' @details We recommend you use \code{sample_file_1213}, rather than \code{sample_file_1314}, unless you need the superannuation variables, 
+#' @details We recommend you use \code{sample_file_1213} over \code{sample_file_1314}, unless you need the superannuation variables, 
 #' as the latter suggests lower-than-recorded tax collections. 
+#' 
+#' Superannuation variables are inflated by a fixed rate of 5\% p.a.
 #' @examples 
-#' if (requireNamespace("taxstats", quietly = TRUE) && requireNamespace("data.table", quietly = TRUE)){
+#' # install.packages('taxstats', repos = 'https://hughparsonage.github.io/drat')
+#' if (requireNamespace("taxstats", quietly = TRUE) &&
+#'     requireNamespace("data.table", quietly = TRUE)) {
 #'   library(taxstats)
 #'   library(data.table)
 #'   sample_file <- copy(sample_file_1314)
-#'   sample_file_1617 <- project(sample_file, h = 3L)  # to "2016-17"
+#'   sample_file_1617 <- project(sample_file,
+#'                               h = 3L, # to "2016-17"
+#'                               fy.year.of.sample.file = "2013-14")  
 #' }
 #' @import data.table
 #' @export
 
 project <- function(sample_file, 
                     h = 0L, 
-                    fy.year.of.sample.file = "2013-14", 
+                    fy.year.of.sample.file = NULL, 
                     WEIGHT = 50L, 
                     excl_vars, 
                     forecast.dots = list(estimator = "mean", pred_interval = 80), 
                     wage.series = NULL,
                     lf.series = NULL,
                     .recalculate.inflators = FALSE, 
-                    .copyDT = TRUE){
+                    .copyDT = TRUE,
+                    check_fy_sample_file = TRUE) {
   stopifnot(is.integer(h), h >= 0L, is.data.table(sample_file))
-  if (.copyDT){
+  if (.copyDT) {
     sample_file <- copy(sample_file)
   }
   
+  if (check_fy_sample_file) {
+    # It's been a common error of mine to switch sample files
+    # without updating the fy.year.of.sample.file
+    if (is.null(fy.year.of.sample.file)) {
+      fy.year.of.sample.file <-
+        match(nrow(sample_file), c(254318L, 258774L, 263339L))
+      if (is.na(fy.year.of.sample.file)) {
+        stop("`fy.year.of.sample.file` was not provided, and its value could not be ",
+             "inferred from nrow(sample_file) = ", nrow(sample_file), ". Either use ", 
+             "a 2% sample file of the years 2012-13, 2013-14, or 2014-15 or ", 
+             "supply `fy.year.of.sample.file` manually.")
+      }
+      fy.year.of.sample.file <- 
+        c("2012-13", "2013-14", "2014-15")[fy.year.of.sample.file]
+    }
+    
+    
+    switch(fy.year.of.sample.file, 
+           "2012-13" = {
+             if (nrow(sample_file) != 254318) {
+               warning("nrow(sample_file) != 254318. Should you choose a different fy.year.of.sample.file?")
+             }
+           },
+           "2013-14" = {
+             if (nrow(sample_file) != 258774) {
+               warning("nrow(sample_file) != 254318. Should you choose a different fy.year.of.sample.file?")
+             }
+           },
+           "2014-15" = {
+             if (nrow(sample_file) != 263339) {
+               warning("nrow(sample_file) != 263339. Should you choose a different fy.year.of.sample.file?")
+             }
+           },
+           stop("`fy.year.of.sample.file` must be '2012-13', '2013-14', or '2014-15'."))
+  }
+  
   sample_file[, "WEIGHT" := list(WEIGHT)]
   
   if (h == 0){
@@ -64,26 +111,29 @@ project <- function(sample_file,
     
     
     cpi.inflator <- cpi_inflator(1, from_fy = current.fy, to_fy = to.fy)
-    if (.recalculate.inflators){
+    if (.recalculate.inflators) {
       CG.inflator <- CG_inflator(1, from_fy = current.fy, to_fy = to.fy)
     } else {
-      if (current.fy %notin% c("2012-13", "2013-14")){
-        stop("Precalculated inflators only available when projecting from 2012-13 or 2013-14.")
+      if (current.fy %notin% c("2012-13", "2013-14", "2014-15")){
+        stop("Precalculated inflators only available when projecting from 2012-13, 2013-14, or 2014-15.")
       } else {
         cg_inflators <- 
           switch(current.fy, 
                  "2012-13" = cg_inflators_1213, 
-                 "2013-14" = cg_inflators_1314)
+                 "2013-14" = cg_inflators_1314,
+                 "2014-15" = cg_inflators_1415)
         stopifnot("forecast.series" %in% names(cg_inflators))
         forecast.series <- NULL 
-        CG.inflator <- cg_inflators[fy_year == to.fy & forecast.series == forecast.dots$estimator][["cg_inflator"]]
+        CG.inflator <- 
+          cg_inflators[fy_year == to.fy] %>%
+          .[forecast.series == forecast.dots$estimator] %>%
+          .subset2("cg_inflator")
       } 
     }
     
     col.names <- names(sample_file)
     
     # Differential uprating not available for years outside:
-    stopifnot(fy.year.of.sample.file %in% yr2fy(2004:2014))
     diff.uprate.wagey.cols <- "Sw_amt"
     
     wagey.cols <- c(
@@ -157,7 +207,8 @@ project <- function(sample_file,
         switch(current.fy, 
                "2012-13" = as.data.table(generic_inflators_1213)[and(fy_year == to.fy, h == H)], 
                "2013-14" = as.data.table(generic_inflators_1314)[and(fy_year == to.fy, h == H)], 
-               stop("Precalculated inflators only available when projecting from 2012-13 or 2013-14."))
+               "2014-15" = as.data.table(generic_inflators_1415)[and(fy_year == to.fy, h == H)], 
+               stop("Precalculated inflators only available when projecting from 2012-13, 2013-14, or 2014-15."))
     }
     
     ## Inflate:
@@ -236,7 +287,8 @@ project <- function(sample_file,
                               Non_emp_spr_amt,
                               Cost_tax_affairs_amt,
                               Other_Ded_amt)] %>%
-      .[, Taxable_Income := pmaxC(Tot_inc_amt - Tot_ded_amt - PP_loss_claimed - NPP_loss_claimed, 0)] 
+      .[, Taxable_Income := pmaxC(Tot_inc_amt - Tot_ded_amt - PP_loss_claimed - NPP_loss_claimed, 0)] %>%
+      .[]
     
   } 
 }
\ No newline at end of file
