@@ -3,7 +3,11 @@
 #' @name student_repayment
 #' 
 #' @param repayment_income The repayment income of the individual, equal to 
-#' \deqn{{Taxable Income} + {Total net investment loss (incl Net rental loss)} + {reportable fringe benefits amounts} + {Reportable super contributions} + {exempt foreign income}}
+#' {Taxable Income} +
+#' {Total net investment loss (incl Net rental loss)} +
+#'  {reportable fringe benefits amounts} +
+#'  {Reportable super contributions} +
+#'  {exempt foreign income}
 #' @param fy.year The financial year \code{repayment_income} was earned.
 #' @param debt The amount of student debt held.
 #' @return The repayment amount.
@@ -41,7 +45,7 @@ student_repayment <- function(repayment_income, fy.year, debt){
                            repayment_threshold = repayment_income, 
                            fy_year = fy.year, debt = debt) %>%
     # to preserve ordering
-    .[, ordering := seq.int(1, .N, by = 1L)] %>%
+    .[, ordering := .I] %>%
     setkeyv(c("fy_year", "repayment_threshold"))
   
   # repayment rate applies to the entire repayment income (not that > threshold, as it is for general tax).
